@@ -6,10 +6,12 @@ from mongoengine.base.document import BaseDocument
 from mongoengine.document import Document
 from rest_framework import serializers
 import sys
+
 if sys.version_info[0] >= 3:
     def unicode(val):
         return str(val)
 
+
 class MongoDocumentField(serializers.WritableField):
     MAX_RECURSION_DEPTH = 5  # default value of depth
 
@@ -50,7 +52,8 @@ class MongoDocumentField(serializers.WritableField):
         Recursion for (embedded) objects
         """
         if depth == 0:
-            return "Max recursion depth exceeded"
+            # Return primary key if exists, else return default text
+            return str(getattr(obj, 'pk', "Max recursion depth exceeded"))
         elif isinstance(obj, BaseDocument):
             # Document, EmbeddedDocument
             return self.transform_document(obj, depth-1)
@@ -60,6 +63,8 @@ class MongoDocumentField(serializers.WritableField):
         elif isinstance(obj, list):
             # List
             return [self.transform_object(value, depth-1) for value in obj]
+        elif obj is None:
+            return None
         else:
             # Default to string
             return unicode(obj)
@@ -115,10 +120,13 @@ class EmbeddedDocumentField(MongoDocumentField):
         return self.to_native(self.default())
 
     def to_native(self, obj):
-        return self.model_field.to_mongo(obj).to_dict()
+        if obj is None:
+            return None
+        else:
+            return self.model_field.to_mongo(obj)
 
-    def from_native(self, obj):
-        return self.model_field.to_python(obj)
+    def from_native(self, value):
+        return self.model_field.to_python(value)
 
 
 class DynamicField(MongoDocumentField):
@@ -126,4 +134,4 @@ class DynamicField(MongoDocumentField):
     type_label = 'DynamicField'
 
     def to_native(self, obj):
-        return self.transform_object(obj, self.depth)
+        return self.model_field.to_python(obj)
